--  ============================================
--	Below Average Items - BelowAverageItems.lua
--	By: gmz323(Greg)
--  ============================================

-- MOP Changes
-- Removal of RangedSlot

-- Constants
local BAI_VERSION = "1.13";

-- Locals
local baiStartTime = GetTime();
local bai_currentAvgLvl = 0;
local bai_slotNameArray = {"BackSlot","ChestSlot","FeetSlot","Finger0Slot","Finger1Slot","HandsSlot","HeadSlot","LegsSlot","MainHandSlot","NeckSlot","SecondaryHandSlot","ShoulderSlot","Trinket0Slot","Trinket1Slot","WaistSlot","WristSlot"};
local bai_slotItems = {}; -- temp table to hold slot item level and if image should be shown
local bai_RefreshAvgLvl = 0;
local HiddenTooltip	= BAI_HiddenTooltip;	--Local pointer

-- Debugging
local debugMode = false;

---============================================================
-- rounds a number to the nearest decimal places
--
local function round(val, decimal)
  if (decimal) then
    return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
  else
    return math.floor(val+0.5)
  end
end

--*****************************************************************************
--*****************************************************************************
SLASH_BAI1 = "/bai"
SLASH_BAI2 = "/belowaverageitems"

SlashCmdList["BAI"] = function(msg, editbox)
	print('Below Average Items - addon version: v' .. BAI_VERSION);
end;

--*****************************************************************************
--*****************************************************************************
local function bai_FindChatWindow(windowname)
	local x;
	
	for x=1,NUM_CHAT_WINDOWS
	do
	  local name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable = GetChatWindowInfo(x);
	  if (name == windowname) then
		return x;
	  end;
	end;
	-- Default to main chat window if DEBUG window not found
	return 1;
end;

--*****************************************************************************
--*****************************************************************************
local function bai_DebugOut(outString)
	if (debugMode) then
		-- calc delta time
		local prefixStr, deltaTime;
		deltaTime = GetTime() - baiStartTime;
		prefixStr = string.format("[%.3f] ", deltaTime);
		-- print to chatframe named "DEBUG" else main chatframe
		getglobal("ChatFrame"..(bai_FindChatWindow("DEBUG"))):AddMessage(prefixStr..outString);
	end;
end;


--*****************************************************************************
--*****************************************************************************
local function bai_UpdateEquippedLvl()

	local total_iLvl, equipped_iLevel = GetAverageItemLevel();
	bai_currentAvgLvl = round(equipped_iLevel,2);
	
end;

--*****************************************************************************
--*****************************************************************************
local function bai_UpdateEquippedItems()
	local i, slotName;
	
	-- Loop through each item of array
	for i,slotName in ipairs(bai_slotNameArray) do 
	
		-- update item table
		bai_slotItems[slotName] = {};
		bai_slotItems[slotName]["indicator"] = 0; -- 0 hide / 1 show
		bai_slotItems[slotName]["iLvl"] = 0; -- slot iLvl, used in tooltip
		getglobal("BAI_"..slotName):Hide();
		
		local slotId, texture, checkRelic = GetInventorySlotInfo(slotName);
		local itemId = GetInventoryItemID("player", slotId);
		
		if (itemId ~= nil) then
			local name, _, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemId);
			
			-- Check for upgraded items if 450+ - workaround
			if (iLevel >= 450) then
				local link = GetInventoryItemLink("player", slotId); -- Thanks ckaotik!
				if (link ~= nil) then
					-- Use HiddenTooltip class to get real item level.
					local HTTiLevel = HiddenTooltip:GetEquipmentItemInfo(link, "ITEM_LEVEL");
					if (HTTiLevel ~= nil) then
						iLevel = tonumber(HTTiLevel);
					end;
				end;
			end;
		
			if (iLevel ~= nil) then
				bai_slotItems[slotName]["iLvl"] = iLevel;
				
				-- Ignore Heirloom items - quality 7
				if (iLevel < bai_currentAvgLvl and quality < 7) then
					bai_slotItems[slotName]["indicator"] = 1;
					getglobal("BAI_"..slotName):Show();
					getglobal("BAI_"..slotName).yellow:Show();
					getglobal("BAI_"..slotName).red:Hide();
				else
					bai_slotItems[slotName]["indicator"] = 0;
					getglobal("BAI_"..slotName):Hide();
					getglobal("BAI_"..slotName).yellow:Hide();
					getglobal("BAI_"..slotName).red:Hide();
				end;
				
			end;
			
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
local function bai_UpdateLowestItems()
	local i, slotName;
	local lowest_iLvl = 9999;
	local bai_tempNameArray = {};
	
	-- Loop through each item of array
	for i,slotName in ipairs(bai_slotNameArray) do 
		-- check if indicator is 1 - slot is below average
		if (bai_slotItems[slotName]["indicator"] == 1) then
			-- check for lowest iLvl
			if (bai_slotItems[slotName]["iLvl"] < lowest_iLvl) then
				lowest_iLvl = bai_slotItems[slotName]["iLvl"];
				bai_tempNameArray = {};
				table.insert(bai_tempNameArray, slotName);
			elseif (bai_slotItems[slotName]["iLvl"] == lowest_iLvl) then
				table.insert(bai_tempNameArray, slotName);
			end;
		end;
	end;

	-- May be multiple lowest level items - loop through and update the graphic from yellow to red
	if (#bai_tempNameArray > 0) then
		for i,slotName in ipairs(bai_tempNameArray) do 
			getglobal("BAI_"..slotName).yellow:Hide();
			getglobal("BAI_"..slotName).red:Show();
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
function BAI_Indicator_OnEnter(self)
	local name = self:GetName();
	
	-- remove "BAI_" from name to get slotname
	local slotName = string.sub(name, 5);
	local iLvl = bai_slotItems[slotName]["iLvl"];
	local displayStr = HIGHLIGHT_FONT_COLOR_CODE .. "Item level - " .. FONT_COLOR_CODE_CLOSE .. RED_FONT_COLOR_CODE .. iLvl .. FONT_COLOR_CODE_CLOSE;
	
	bai_DebugOut("BAI_Indicator_OnEnter: ".. name);
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:AddLine(displayStr);
	displayStr = HIGHLIGHT_FONT_COLOR_CODE .. "Below average of " .. FONT_COLOR_CODE_CLOSE .. GREEN_FONT_COLOR_CODE .. bai_currentAvgLvl .. FONT_COLOR_CODE_CLOSE;
	GameTooltip:AddLine(displayStr);
	GameTooltip:Show();
	
end;

--*****************************************************************************
--*****************************************************************************
function BAI_OnShow(self)
	local name = self:GetName();
	
	bai_DebugOut("BAI_OnShow: ".. name);

	if (bai_RefreshAvgLvl == 1) then
		bai_UpdateEquippedLvl();
		bai_UpdateEquippedItems();
		bai_UpdateLowestItems();
		bai_RefreshAvgLvl = 0;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function BAI_OnEvent(self, event, ...)

	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
	
	-- Things to do after addon is loaded
	if (event == "ADDON_LOADED") then
		if (arg1 == "BelowAverageItems") then
			bai_DebugOut("BAI_OnEvent: ".. arg1 .. " " .. event);
			-- current iLvl not available at this point - so hold off to OnShow
		end;
	end;
	
	-- Equpment changed - recheck for below average gear
	if (event == "PLAYER_EQUIPMENT_CHANGED") then
		local slot, hasItem = ...;
		if hasItem == nil then 
			hasItem = 0;
		end;
		bai_DebugOut(event .. " slot: " .. slot .. " hasItem: " .. hasItem);
		-- update current iLvl average
		bai_UpdateEquippedLvl();
		-- update current items
		bai_UpdateEquippedItems();
		-- update lowest items
		bai_UpdateLowestItems();
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
function BAI_OnLoad(self)

	bai_DebugOut("BAI_OnLoad: "..self:GetName());
	
	-- Register Events
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	
	-- init variables
	bai_RefreshAvgLvl = 1;
	
end;